/*===================================================================================*/
/* Rohde & Schwarz Directional Power Sensor NRT-Z 			 						 */
/* Application Example 																 */
/*																					 */       
/* MODIFICATON HISTORY       														 */
/* Ver. 1.7, 12/2003, CVI 5.0.1 	                                                 */  
/*	       changed for nrtz driver Revision 1.4.x									 */
/*		by Werner Klotzsche, Rohde & Schwarz										 */   
/*																					 */
/* Ver. 1.6, 08/10/1999, CVI 5.0.1	                                                 */
/*===================================================================================*/
#include <utility.h>
#include "f:\treiber test\nrtz\rsnrtz.h"
#include <ansi_c.h>
#include <cvirte.h>   /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "rsnrtz_CVI_example.h"

//#define NOHW
#define DATA_SIZE 100 // 2 * 50 points per chart = 2 * history

static int panelHdl, initHdl, zeroingHdl, helpHdl, aboutHdl; // panel handles
static int unit, powerfwd, reflection; // caching of selected values
static ViSession instrHdl = -1; // instrument handle
static double fwdMin = 1e9, fwdMax = -1e9, revMin = 1e9, revMax = -1e9; // "autoscaling" ranges
static double data[DATA_SIZE], *p2data, *p2datar; // history buffer for chart & pointer to actual point
static double fwdFixMin, fwdFixMax, revFixMin, revFixMax; // fixscaling ranges
static int fwdScaling, revScaling;

// function declarations
int CVICALLBACK IdleEventCallback (int panelOrMenuBarHandle, int controlOrMenuItemID,
                                  int event, void *callbackData, int eventData1,
                                  int eventData2);
void processMeasuredValue(double *fwd, double *rev, int unit, int powerfwd, int reflection);

// is invoked when program starts
int main (int argc, char *argv[])
{
  // initialize
  if (InitCVIRTE (0, argv, 0) == 0)
    return -1;  /* out of memory */
  if ((panelHdl = LoadPanel (0, "rsnrtz_CVI_example.uir", PANEL)) < 0)
    return -1;
  if ((initHdl = LoadPanel (0, "rsnrtz_CVI_example.uir", INIT)) < 0)
    return -1;
  if ((zeroingHdl = LoadPanel (0, "rsnrtz_CVI_example.uir", ZEROING)) < 0)
    return -1;
  if ((helpHdl = LoadPanel (0, "rsnrtz_CVI_example.uir", HELP)) < 0)
    return -1;
  if ((aboutHdl = LoadPanel (0, "rsnrtz_CVI_example.uir", ABOUT)) < 0)
    return -1;
  GetCtrlVal (panelHdl, PANEL_SCALING1, &fwdScaling);
  GetCtrlVal (panelHdl, PANEL_SCALING2, &revScaling);
  GetCtrlVal (panelHdl, PANEL_MIN1, &fwdFixMin);
  GetCtrlVal (panelHdl, PANEL_MAX1, &fwdFixMax);
  GetCtrlVal (panelHdl, PANEL_MIN2, &revFixMin);
  GetCtrlVal (panelHdl, PANEL_MAX2, &revFixMax);
  DisplayPanel (panelHdl);
  RunUserInterface ();
  return 0;
}

// invoked when Configure button pressed
int CVICALLBACK configureCall (int panel, int control, int event,
    void *callbackData, int eventData1, int eventData2)
{
  int i1, i2, i3;
  short sh;
  double dbl, dbl2, dbl3;
  switch (event)
    {
    case EVENT_COMMIT:
      InstallMainCallback (0, 0, 1);
      InstallPopup(initHdl);
      ClearStripChart (panelHdl, PANEL_CHART1);
      ClearStripChart (panelHdl, PANEL_CHART2);
      fwdMin = 1e9; fwdMax = -1e9; revMin = 1e9; revMax = -1e9;
      p2data = data;
      
      // do configuration
#ifndef NOHW
      rsnrtz_configMeas (instrHdl, VI_OFF, 0, &sh, &sh);
#else
      Delay(3);
#endif
      
      GetCtrlVal (panelHdl, PANEL_FREQUENCY, &dbl);
#ifndef NOHW
      rsnrtz_configCarrFreq (instrHdl, dbl, &dbl2);
#endif
      
      GetCtrlVal (panelHdl, PANEL_AVERAGINGMODE, &i1);
      GetCtrlVal (panelHdl, PANEL_FILTERCOUNT, &i3);
#ifndef NOHW
	  rsnrtz_configAver (instrHdl, i1, i3, VI_OFF, &sh, &sh);
#endif
      
      GetCtrlVal (panelHdl, PANEL_DIRECTION, &i1);
#ifndef NOHW
      rsnrtz_configDir (instrHdl, i1, &sh);
#endif
      
      GetCtrlVal (panelHdl, PANEL_MEASPOSITION, &i1);
#ifndef NOHW
      rsnrtz_configRefPlane (instrHdl, i1, &sh);
#endif
      
      GetCtrlVal (panelHdl, PANEL_INTEGRATIONTIME, &dbl);
#ifndef NOHW
      rsnrtz_configIntTime (instrHdl, dbl, &dbl2);
#endif

      GetCtrlVal (panelHdl, PANEL_VIDEOBANDWIDTH, &i1);
#ifndef NOHW
      rsnrtz_configVideoBW (instrHdl, i1, VI_OFF, &dbl, &sh);
#endif
      
      GetCtrlVal (panelHdl, PANEL_CCDFTHRESHOLD, &dbl);
#ifndef NOHW
      rsnrtz_configCCDF (instrHdl, dbl, &dbl2);
#endif

      GetCtrlVal (panelHdl, PANEL_BURSTPERIOD, &dbl);
      GetCtrlVal (panelHdl, PANEL_BURSTWIDTH, &dbl2);
#ifndef NOHW
      rsnrtz_configBurst (instrHdl, dbl, dbl2, &dbl3, &dbl3);
#endif
      
      GetCtrlVal (panelHdl, PANEL_POWERUNIT, &unit);
      GetCtrlVal (panelHdl, PANEL_POWERFWD, &powerfwd);
#ifndef NOHW
      rsnrtz_actstatFwdMeasMode (instrHdl, powerfwd, &sh);
#endif
      // set units
      switch (powerfwd) {
        case 0: case 1: case 4: case 5:
          SetCtrlVal (panelHdl, PANEL_POWERFWDUNIT, unit?"W":"dBm"); break;
        case 2: // %
          SetCtrlVal (panelHdl, PANEL_POWERFWDUNIT, "%"); break;
        case 3: // dB
          SetCtrlVal (panelHdl, PANEL_POWERFWDUNIT, "dB"); break;
      }
      
      GetCtrlVal (panelHdl, PANEL_REFLECTION, &reflection);
	  
	  
	  SetCtrlAttribute (panelHdl, PANEL_CHART2, ATTR_YFORMAT,
						(reflection == 3) ? 0xFFF0 : 0xFFF2);
	  SetCtrlAttribute (panelHdl, PANEL_CHART2, ATTR_YPRECISION,
						(reflection == 3) ? 2 : 1);
	  SetCtrlAttribute (panelHdl, PANEL_REFLECTIONIND, ATTR_FORMAT,
						(reflection == 3) ? VAL_FLOATING_PT_FORMAT : VAL_ENGINEERING_FORMAT);

#ifndef NOHW
      rsnrtz_actstatRevMeasMode (instrHdl, reflection, &sh);
#endif
      switch (reflection) {
        case 0:
          SetCtrlVal (panelHdl, PANEL_REFLECTIONUNIT, unit?"W":"dBm");
          SetCtrlAttribute (panelHdl, PANEL_MIN2, ATTR_CHECK_RANGE,  VAL_IGNORE);
          break;
        case 1:
          SetCtrlVal (panelHdl, PANEL_REFLECTIONUNIT, "");
          SetCtrlAttribute (panelHdl, PANEL_MIN2, ATTR_CHECK_RANGE,  VAL_IGNORE);
          break;
        case 2: // dB
          SetCtrlVal (panelHdl, PANEL_REFLECTIONUNIT, "dB");
          SetCtrlAttribute (panelHdl, PANEL_MIN2, ATTR_CHECK_RANGE,  VAL_IGNORE);
          break;
        case 3: // SWR
          SetCtrlVal (panelHdl, PANEL_REFLECTIONUNIT, "");
          SetCtrlAttribute (panelHdl, PANEL_MIN2, ATTR_CHECK_RANGE, VAL_COERCE);
          SetCtrlAttribute (panelHdl, PANEL_MIN2, ATTR_MIN_VALUE, 1.0);
          GetCtrlVal (panelHdl, PANEL_MIN2, &revFixMin);
          if (revFixMin > revFixMax) {
              SetCtrlVal (panelHdl, PANEL_MAX2, revFixMax = revFixMin);
          }
          if (!revScaling) {
              SetCtrlAttribute (panelHdl, PANEL_MIN2, ATTR_DIMMED, 0);
              SetCtrlAttribute (panelHdl, PANEL_MAX2, ATTR_DIMMED, 0);
              SetAxisScalingMode (panelHdl, PANEL_CHART2, VAL_LEFT_YAXIS,
                  VAL_MANUAL, revFixMin, (revFixMin < revFixMax)?revFixMax:(revFixMax + 1e-3));
          }
          break;
      }
      InstallMainCallback (IdleEventCallback, 0, 1);
      SetIdleEventRate (100);
      RemovePopup (1);
      break;
    case EVENT_RIGHT_CLICK:
            SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Configure");
      ResetTextBox (helpHdl, HELP_TEXTBOX, "Pushing this button will configure the NRT-Z according to the current panel settings.");
          SetCtrlAttribute (helpHdl, HELP_TEXTBOX, ATTR_FIRST_VISIBLE_LINE, 0);
          DisplayPanel(helpHdl);
          break;
    }
  return 0;
}

// invoked when Zero button pressed
int CVICALLBACK zeroCall (int panel, int control, int event,
    void *callbackData, int eventData1, int eventData2)
{
  double d;
  switch (event)
    {
    case EVENT_COMMIT:
      // do zeroing
      InstallPopup (zeroingHdl);
#ifndef NOHW
      rsnrtz_actstatZero (instrHdl, VI_ON, &d, &d, &d, &d, &d);
#else
      Delay(3);
#endif
      RemovePopup (1);
      break;
    case EVENT_RIGHT_CLICK:
            SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Zero");
      ResetTextBox (helpHdl, HELP_TEXTBOX, "Pushing this button will trigger zeroing of the NRT-Z.");
          SetCtrlAttribute (helpHdl, HELP_TEXTBOX, ATTR_FIRST_VISIBLE_LINE, 0);
          DisplayPanel(helpHdl);
          break;
    }
  return 0;
}

// invoked when Run/Quit button pressed
int CVICALLBACK quitCall (int panel, int control, int event,
    void *callbackData, int eventData1, int eventData2)
{
  switch (event)
    {
    case EVENT_COMMIT:
      if (instrHdl == -1) { // Run pressed
        char resStr[50];
        int port;
        GetCtrlVal(panelHdl, PANEL_COMPORT, &port);
        sprintf(resStr, "ASRL%d::INSTR", port);
        // init instrument
#ifndef NOHW
        if (rsnrtz_init (resStr, VI_ON, VI_OFF, &instrHdl) != 0) {
          // init failed
          instrHdl = -1;
          SetCtrlVal(panelHdl, PANEL_QUIT, 0);
        } else {
#else
          instrHdl = 1;
#endif
		 {
			double  min = 1.0,
			        max = 300.0;
			ViChar	buffer[512];
			ViInt32	nb;
			       
#ifndef NOHW
			rsnrtz_writeInstrData (instrHdl, "ID\r\n");
			rsnrtz_readInstrData (instrHdl, 512, buffer, &nb);
			min = strstr(buffer, "NRT-Z44") ? 1.0 : 0.25;
    		max = strstr(buffer, "NRT-Z44") ? 300.0 : 75.0;
#endif
		 SetCtrlAttribute (panelHdl, PANEL_CCDFTHRESHOLD, ATTR_MIN_VALUE, min);
		 SetCtrlAttribute (panelHdl, PANEL_CCDFTHRESHOLD, ATTR_MAX_VALUE, max);
		 }

          // init OK
          SetCtrlAttribute(panelHdl, PANEL_ZERO, ATTR_DIMMED, 0);
          SetCtrlAttribute(panelHdl, PANEL_CONFIGURE, ATTR_DIMMED, 0);
          SetCtrlAttribute(panelHdl, PANEL_SCALING1, ATTR_DIMMED, 0);
          SetCtrlAttribute(panelHdl, PANEL_SCALING2, ATTR_DIMMED, 0);
          configureCall(panelHdl, PANEL_CONFIGURE, EVENT_COMMIT, 0, 0, 0);
#ifndef NOHW
          InstallMainCallback (IdleEventCallback, 0, 1); // measurement callback
        }
#endif
      } else { // Quit pressed
        InstallMainCallback (0, 0, 1);
#ifndef NOHW
        rsnrtz_close (instrHdl);
#endif
        instrHdl = -1;
        QuitUserInterface (0);
      }
      break;
    case EVENT_RIGHT_CLICK:
            SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Run / Quit");
      ResetTextBox (helpHdl, HELP_TEXTBOX, "Pushing this button will run or terminate the application example.");
          SetCtrlAttribute (helpHdl, HELP_TEXTBOX, ATTR_FIRST_VISIBLE_LINE, 0);
          DisplayPanel(helpHdl);
          break;
    }
  return 0;
}

// window closing callback
int CVICALLBACK panelCall (int panel, int event, void *callbackData,
    int eventData1, int eventData2)
{
  if (event == EVENT_CLOSE) {
    if (panel == panelHdl) {
      // quit application
      if (instrHdl != -1) {
        InstallMainCallback (0, 0, 1);
#ifndef NOHW
        rsnrtz_close (instrHdl);
#endif
        instrHdl = -1;
      }
      QuitUserInterface (0);
    } else if (panel == helpHdl) {
      HidePanel(panel);
    }
  }
  return 0;
}

// measurement callback - invoked as often as possible
int CVICALLBACK IdleEventCallback (int panelOrMenuBarHandle,
                                  int controlOrMenuItemID,
                                  int event,
                                  void *callbackData,
                                  int eventData1,
                                  int eventData2) {
  static int changed = 0, status;
  double fwd, rev;
  if (instrHdl == -1) return 0; // not initialized
  
  // do measurement
#ifndef NOHW
  rsnrtz_actstatMeas (instrHdl, VI_OFF, &fwd, &rev);
  processMeasuredValue(&fwd, &rev, unit, powerfwd, reflection);
#else
    fwd = 2.0 * rand() / (double)RAND_MAX - 1.0; rev = 2.0 * rand() / RAND_MAX - 1.0;
#endif
  
  // this part of code solves displaying and "autoscaling" in strip chart
  if (p2data - data >= DATA_SIZE) {
    memmove (data, data + 2, (DATA_SIZE-2)*sizeof(double));
    p2data -= 2;
  }

  *p2data = fwd; p2data++;
  *p2data = rev; p2data++;
  
  if (fwdScaling) {
		double* p2dbl;
		fwdMin = 1e9; fwdMax = -1e9;
		for (p2dbl = data; p2dbl < p2data; p2dbl+=2) {
			if (fwdMin > *p2dbl) {
				fwdMin = *p2dbl;
			}
			if (fwdMax < *p2dbl) {
				fwdMax = *p2dbl;
			}
		}
		SetAxisScalingMode (panelHdl, PANEL_CHART1, VAL_LEFT_YAXIS,
			VAL_MANUAL, fwdMin, (fwdMin < fwdMax)?fwdMax:(fwdMax + 1e-3));
		PlotStripChart (panelHdl, PANEL_CHART1, data, (p2data - data) / 2, 0, 1,
			VAL_DOUBLE);
	}
	else {
		PlotStripChartPoint (panelHdl, PANEL_CHART1, fwd);
  }
  
  if (revScaling) {
		double* p2dbl;
		revMin = 1e9; revMax = -1e9;
		
		for (p2dbl = data+1; p2dbl < p2data-1; p2dbl+=2) {
			if (revMin > *p2dbl) {
				revMin = *p2dbl;
			}
			if (revMax < *p2dbl) {
				revMax = *p2dbl;
			}
		}
		if (revMin > revMax)
			revMin = revMax;
		SetAxisScalingMode (panelHdl, PANEL_CHART2, VAL_LEFT_YAXIS,
			VAL_MANUAL, revMin, (revMin < revMax)?revMax:(revMax + 1e-3));
		PlotStripChart (panelHdl, PANEL_CHART2, data, (p2data - data) / 2, 1, 1,
			VAL_DOUBLE);
	}
	else {
    	PlotStripChartPoint (panelHdl, PANEL_CHART2, rev);
  }
  // end of solving "autoscaling" in strip chart
  SetCtrlVal (panelHdl, PANEL_POWERFWDIND, fwd);
  SetCtrlVal (panelHdl, PANEL_REFLECTIONIND, rev);
  return 0;
}

// for processing of measured values (depends on selected measurement mode)
void processMeasuredValue(double *fwd, double *rev, int unit, int powerfwd, int reflection) {
  switch (powerfwd) {
    case 0: case 1: case 4: case 5:
      if(!unit) {
        *fwd = fabs(*fwd / 0.001);
        *fwd = 10.0 * log10 (*fwd);
      } break;
    case 3: *fwd = 10.0 * log10 (fabs(*fwd)); break;
  }
  
  if (reflection == 0) {
    if(!unit) {
      *rev = fabs(*rev / 0.001);
      *rev = 10.0 * log10 (*rev);
    }
  }
}

// for displaing help panel
int CVICALLBACK helpCall (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_RIGHT_CLICK) {
        switch (control) {
          case PANEL_POWERFWD:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Power (FWD)");
        ResetTextBox (helpHdl, HELP_TEXTBOX, 
        "This control selects the forward measurement function.\n\n"

        "Valid Range:\n"
        "0 - AVG (Default Value)\n"
        "1 - Av. Burst (User)\n"
        "2 - CCDF\n"
        "3 - CF\n"
        "4 - Av. Burst (Auto)\n"
        "5 - PEP\n\n"

        "Notes:\n\n"

        "(1) Average = measurement of average forward power. This measurement function provides the lowest measurement uncertainty and the widest measurement range.\n\n"

        "(2) Av. Burst (User) = calculated burst average. Determines the average power of modulated or unmodulated RF bursts. The CBAV value is calculated from the average power and the duty cycle.\n\n"

        "(3) CCDF = complementary cumulative distribution function. This measurement function indicates the probability in % of the forward envelope power exceeding a preset threshold (see function Config CCDF).\n\n"

        "(4) CF = crest factor. CF is the ration of peak envelope power (PEP) to average power (Average). The CF provides information on the amount of modulation distortion and may assume very high values in the case"
        "of burst or spread-spectrum signals.\n\n"

        "(5) Av. Burst (Auto) = measured burst average. The MBAV is calculated from the average power and the duty cycle.\n\n"

        "(6) PEP = peak envelope power. PEP function is a continuous measurement of peak envelope power of modulated signals and bursts.\n\n"

        "(7) For CCDF, CF, Av. Burst (Auto), and PEP the power sensor must be connected for the forward power in the privileged direction to ensure proper functionality.\n\n");
              break;
            case PANEL_REFLECTION:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Reflection");
        ResetTextBox (helpHdl, HELP_TEXTBOX,
        "This control selects the reverse measurement mode.\n\n"

        "Valid Range:\n"
        "1 - R. CO\n"
        "2 - RL (Default Value)\n"
        "3 - SWR\n\n"

        "(1) R. CO = Reflection Coefficient, defined as the ratio of reflected to incident wave.\n\n"

        "(2) RL = Return Loss. RL indicates the attenuation (in dB) of the reflected wave relative to the incident wave.\n\n"

        "(3) SWR = Standing Wave Ratio. SWR indicates the ration of maximum to minimum voltage on an RF line.\n\n"
        );
        break;
      case PANEL_POWERUNIT:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Power Unit");
        ResetTextBox (helpHdl, HELP_TEXTBOX,
        "Specifies power unit for both forward and reverse measurement.\n\n"

        "Valid Range:\n"
        "0 - dBm\n"
        "1 - W\n"
        );
        break;
      case PANEL_BURSTPERIOD:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Burst Period");
        ResetTextBox (helpHdl, HELP_TEXTBOX,
        "This control specifies the burst period.\n\n"

        "Valid Range: <Burst Width> to 1.0 s\n\n"

        "Default Value: 0.01 s"
        );
        break;
      case PANEL_BURSTWIDTH:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Burst Width");
        ResetTextBox (helpHdl, HELP_TEXTBOX,
        "This control specifies the burst width.\n\n"

        "Valid Range: 1.0E-9 s to <Burst Period>\n\n"

        "Default Value: 1.0 ms"
        );
        break;
      case PANEL_CCDFTHRESHOLD:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - CCDF Threshold");
        ResetTextBox (helpHdl, HELP_TEXTBOX,
        "This control specifies the CCDF threshold.\n\n"

        "Valid Range:\n\n"

        "NRT-Z43: 0.25 W to 75.0 W\n"
        "NRT-Z44: 1.0  W to 300.0 W\n\n"

        "Default Value: 1.0 W"
        );
        break;
      case PANEL_FILTERCOUNT:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Filter Count");
        ResetTextBox (helpHdl, HELP_TEXTBOX,
        "This control specifies fixed count of averaging filters.\n\n"

        "Valid Range:\n"
        "0 - 1 (Default Value)\n"
        "1 - 2\n"
        "2 - 4\n"
        "3 - 8\n"
        "4 - 16\n"
        "5 - 32\n"
        "6 - 64\n"
        "7 - 128\n"
        "8 - 256 (only NRT-Z43)\n\n"

        "Note:\n\n"

        "(1) This control is active only with Averaging Mode set to User.\n\n"

        "(2) Filter count of 1 means that no averaging takes place and each measured value is output immediately."
        );
        break;
      case PANEL_VIDEOBANDWIDTH:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Video Bandwidth");
        ResetTextBox (helpHdl, HELP_TEXTBOX,
        "This control specifies the passband (starting at DC) of the forward channel for the demodulated signal. The video bandwidth should be selected as small as possible to obtain stable results even at low "
        "forward powers (noice reduction). It should not be selected too small, however, as otherwise significant distortion of the demolutaded signal may result.\n\n"

        "Valid Range:\n"
        "0 - 4 kHz\n"
        "1 - 200 kHz (Default Value)\n"
        "2 - 4 MHz"
        );
        break;
      case PANEL_INTEGRATIONTIME:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Integration Time");
        ResetTextBox (helpHdl, HELP_TEXTBOX,
        "This control sets the integration time. This setting has effect on stability of results at low powers (noise) and also with low frequency modulation of the envelope. The default value of 0.037 s, on which all specifications are based, results "
        "in suppression of low-frequency modulation as from 20 Hz and in optimum noise reduction. Integration times shorter then 0.037 s allow higher measurement speed but at the same time increase the noise component if low power levels are measured."
        " Values below 10 ms are very likely to result in a strong increase of noise. Integration times longer then 0.037 s allow measurements to be made with low-frequency modulation but will not reduce noise.\n\n"

        "Valid Range: 5.0 ms to 0.111 s\n\n"

        "Default Value: 0.03667 s"
        );
        break;
      case PANEL_DIRECTION:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Direction");
        ResetTextBox (helpHdl, HELP_TEXTBOX,
        "This control specifies the direction of the forward power relative to the sensor ports 1 and 2.\n\n"

        "Valid Range:\n"
        "0 - Auto (Default Value)\n"
        "1 - 1>2\n"
        "2 - 2>1\n\n"

        "Notes:\n\n"

        "(1) In the Auto mode the assignment is made automatically by the power sensor, which interprets the greater of the two measured powers as forward power. The Auto mode may not be advisable if the forward and the reverse power are approx. "
        "equal.\n\n"

        "(2) 1>2: The wave from port 1 to port 2 is defined as the forward wave. 2>1: The wave from port 2 to port 1 is defined as the forward wave.\n\n"

        "(3) In the normal case the power sensor should be inserted in the test circuit such that the source is connected to port 1 and the load to port 2."
        );
        break;
      case PANEL_FREQUENCY:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Carrier Frequency");
        ResetTextBox (helpHdl, HELP_TEXTBOX,
        "This control specifies the carrier frequency of the test signal.\n\n"

        "Valid Range:\n\n"

        "NRT-Z43: 400.0 MHz to 4.0 GHz\n"
        "NRT-Z44: 200.0 MHz to 4.0 GHz\n\n"

        "Default Value: 1.0 GHz"
        );
        break;
      case PANEL_AVERAGINGMODE:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Averaging Mode");
        ResetTextBox (helpHdl, HELP_TEXTBOX,
        "This control selects the averaging mode. In the mode Auto the count of the averaging filter is selected as a function of the forward power, "
        "the test function and the desired resolution of results (Averaging Resolution). The smaller the power measured, ie the higher the "
        "relative percentage of random measured value variations, and the higher the resolution, the greater the number of measured values "
        "averaged. Non-averaged measured values will be obtained in the Auto mode only with low Averaging Resolution = Low and at comaratively high powers.\n\n"
        
        "Valid Range:\n"
        "F - Auto (Default Value)\n"
        "T - User\n\n"

        "Note:\n\n"

        "(1) If set to User the Averaging Count becomes active."
        );
        break;
      case PANEL_MEASPOSITION:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Measurement Position");
        ResetTextBox (helpHdl, HELP_TEXTBOX,
        "This controls defines the reference plane for the measurement. Selection of appropriate reference plane is necessary if more exact demands are made on measurement accuracy.\n\n"

        "Valid Range:\n"
        "F - Load (Default Value)\n"
        "T - Source"
        );
        break;
            case PANEL_COMPORT:
        SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - COM Port");
        ResetTextBox (helpHdl, HELP_TEXTBOX,
              "Specifies COM port to which the NRT-Z is connected.\n\n"

        "Valid Range:\n"
        "0 - COM1 (Default Value)\n"
        "1 - COM2\n"
        "2 - COM3\n"
        "3 - COM4\n"
        "4 - COM5"
        );
        break;
            default:
              SetPanelAttribute (helpHdl, ATTR_TITLE, "Help");
        ResetTextBox (helpHdl, HELP_TEXTBOX, "No help available.");
        }
        SetCtrlAttribute (helpHdl, HELP_TEXTBOX, ATTR_FIRST_VISIBLE_LINE, 0);
        DisplayPanel(helpHdl);
  }
    return 0;
}

// for hiding help panel
int CVICALLBACK helpCloseCall (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT) {
        HidePanel(helpHdl);
  }
    return 0;
}

// about box callback
void CVICALLBACK AboutCall (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    InstallPopup (aboutHdl);
}

// about OK callback
int CVICALLBACK AboutOkCall (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            RemovePopup (0);
            break;
        }
    return 0;
}

int CVICALLBACK scalingCall (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        switch (control) {
            case PANEL_SCALING1:
            case PANEL_MIN1:
            case PANEL_MAX1:
                GetCtrlVal (panelHdl, PANEL_SCALING1, &fwdScaling);
                GetCtrlVal (panelHdl, PANEL_MIN1, &fwdFixMin);
                GetCtrlVal (panelHdl, PANEL_MAX1, &fwdFixMax);
                if (fwdFixMin > fwdFixMax) {
                    SetCtrlVal (panelHdl, PANEL_MAX1, fwdFixMax = fwdFixMin);
                }
                if (!fwdScaling) {
                    SetCtrlAttribute (panelHdl, PANEL_MIN1, ATTR_DIMMED, 0);
                    SetCtrlAttribute (panelHdl, PANEL_MAX1, ATTR_DIMMED, 0);
                    SetAxisScalingMode (panelHdl, PANEL_CHART1, VAL_LEFT_YAXIS,
                        VAL_MANUAL, fwdFixMin, (fwdFixMin < fwdFixMax)?fwdFixMax:(fwdFixMax + 1e-3));
                } else {
                    // initialize auto scaling
                    double* p2dbl;
                    SetCtrlAttribute (panelHdl, PANEL_MIN1, ATTR_DIMMED, 1);
                    SetCtrlAttribute (panelHdl, PANEL_MAX1, ATTR_DIMMED, 1);
                    fwdMin = 1e9; fwdMax = -1e9;
                    for (p2dbl = data; p2dbl < p2data; p2dbl+=2) {
                        if (fwdMin > *p2dbl) {
                            fwdMin = *p2dbl;
                        }
                        if (fwdMax < *p2dbl) {
                            fwdMax = *p2dbl;
                        }
                    }
                    SetAxisScalingMode (panelHdl, PANEL_CHART1, VAL_LEFT_YAXIS,
                            VAL_MANUAL, fwdMin, (fwdMin < fwdMax)?fwdMax:(fwdMax + 1e-3));
                }
                PlotStripChart (panelHdl, PANEL_CHART1, data, (p2data - data) / 2, 0, 1,
                            VAL_DOUBLE);
                break;
            case PANEL_SCALING2:
            case PANEL_MIN2:
            case PANEL_MAX2:
                GetCtrlVal (panelHdl, PANEL_SCALING2, &revScaling);
                GetCtrlVal (panelHdl, PANEL_MIN2, &revFixMin);
                GetCtrlVal (panelHdl, PANEL_MAX2, &revFixMax);
                if (revFixMin > revFixMax) {
                    SetCtrlVal (panelHdl, PANEL_MAX2, revFixMax = revFixMin);
                }
                if (!revScaling) {
                    SetCtrlAttribute (panelHdl, PANEL_MIN2, ATTR_DIMMED, 0);
                    SetCtrlAttribute (panelHdl, PANEL_MAX2, ATTR_DIMMED, 0);
                    SetAxisScalingMode (panelHdl, PANEL_CHART2, VAL_LEFT_YAXIS,
                        VAL_MANUAL, revFixMin, (revFixMin < revFixMax)?revFixMax:(revFixMax + 1e-3));
                } else {
                    // initialize auto scaling
                    double* p2dbl;
                    SetCtrlAttribute (panelHdl, PANEL_MIN2, ATTR_DIMMED, 1);
                    SetCtrlAttribute (panelHdl, PANEL_MAX2, ATTR_DIMMED, 1);
                    revMin = 1e9; revMax = -1e9;
                    for (p2dbl = data+1; p2dbl < p2data-1; p2dbl+=2) {
                        if (revMin > *p2dbl) {
                            revMin = *p2dbl;
                        }
                        if (revMax < *p2dbl) {
                            revMax = *p2dbl;
                        }
                    }
                    SetAxisScalingMode (panelHdl, PANEL_CHART2, VAL_LEFT_YAXIS,
                            VAL_MANUAL, revMin, (revMin < revMax)?revMax:(revMax + 1e-3));
                }
                PlotStripChart (panelHdl, PANEL_CHART2, data, (p2data - data) / 2, 1, 1,
                        VAL_DOUBLE);
                break;
        }
    } else if (event == EVENT_RIGHT_CLICK) {
        switch (control) {
          case PANEL_SCALING1:
          case PANEL_SCALING2:
            SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Scaling");
            ResetTextBox (helpHdl, HELP_TEXTBOX, 
            "This control selects the graph scaling mode.\n\n"

            "Valid Range:\n"
            "VI_ON  (1) - Auto  (Default Value)\n"
            "VI_OFF (0) - Fix\n\n"

            "Notes:\n\n"

            "(1) Fix = you can set the range manually\n\n");
            break;
          case PANEL_MIN1:
          case PANEL_MIN2:
            SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Minimum");
            ResetTextBox (helpHdl, HELP_TEXTBOX, 
            "This control sets the Y axis minimum for Fix scaling mode.\n\n");
            break;
          case PANEL_MAX1:
          case PANEL_MAX2:
            SetPanelAttribute (helpHdl, ATTR_TITLE, "Help - Maximum");
            ResetTextBox (helpHdl, HELP_TEXTBOX, 
            "This control sets the Y axis maximum for Fix scaling mode.\n\n");
            break;
          default:
              SetPanelAttribute (helpHdl, ATTR_TITLE, "Help");
              ResetTextBox (helpHdl, HELP_TEXTBOX, "No help available.");
        }
        SetCtrlAttribute (helpHdl, HELP_TEXTBOX, ATTR_FIRST_VISIBLE_LINE, 0);
        DisplayPanel(helpHdl);
    }
        
    return 0;
}
